#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "common.h"
#include "macros.h"
#include "failures.h"
#include "vm.h"
#include "vmlibcall.h"
#include "vmlibcalln.h"

static void do_special(char **cur, int *ret)
{
	char *ch, *pos;

	pos = *cur;
	while ((ch = strchr(pos, '\\')) != NULL) {

		*ch = 0;
		*ret += printf("%s", pos);
		switch (ch[1]) {
		case 'n':
			*ret += printf("\n");
			break;
		case 't':
			*ret += printf("\t");
			break;
		case '\\':
			*ret += printf("\\");
			break;
		}
		ch += 2;
		pos = ch;
	}
	*cur = pos;
}

static void vm_printf(struct vm *machine)
{
	char fmt_str[STRING_LEN];
	char str[STRING_LEN];
	char *ch, *pos;
	int ret;

	vm_stack_get_string(machine, fmt_str);

	ret = 0;
	pos = fmt_str;
	while ((ch = strchr(pos, '%')) != NULL) {
		*ch = 0;

		do_special(&pos, &ret);
		ret += printf("%s", pos);

		switch (ch[1]) {
		case 'd':
		case 'i':
			ret += printf("%d", vm_stack_get_int(machine));
			break;
		case 'x':
			ret += printf("%x", vm_stack_get_int(machine));
			break;
		case 'f':
			ret += printf("%f", vm_stack_get_float(machine));
			break;
		case 'c':
			ret += printf("%c", vm_stack_get_char(machine));
			break;
		case 's':
			vm_stack_get_string(machine, str);
			ret += printf("%s", str);
			break;
		case '%':
			ret += printf("%%");
			break;
		}
		ch += 2;
		pos = ch;
	}

	do_special(&pos, &ret);
	ret += printf("%s", pos);

	vm_stack_poke_int(machine, ret);
}

static struct vm_libcall libcalls[] = {
	{ .name = "printf",	.num = VM_PRINTF, .handler = vm_printf },
	{ .name = NULL,		.num = 0,	  .handler = NULL }
};

void vm_libcall_init()
{
	unsigned int i;
	ret_t ret;

	for (i = 0; libcalls[i].name != NULL; i++) {
		ret = vm_libcall_register(&libcalls[i]);
		if (ret != ret_ok)
			xerror(1, "can't register `%s' library call",
					libcalls[i].name);
	}
}

